@@ -36,7 +36,7 @@ basic_types = (['NUMBER', ['INT8', 'INT4', 'INT2', 'FLOAT8', 'FLOAT4',
                ['INTEGER', ['INT4', 'INT2']],
                ['FLOAT', ['FLOAT8', 'FLOAT4', 'NUMERIC']],
                ['STRING', ['NAME', 'CHAR', 'TEXT', 'BPCHAR',
-                           'VARCHAR', 'LZTEXT']],
+                           'VARCHAR']],
                ['BOOLEAN', ['BOOL']],
                ['DATETIME', ['DATE', 'TIME', 'TIMETZ', 
                              'TIMESTAMP', 'TIMESTAMPTZ', 
