@@ -21,7 +21,7 @@
 
 /*
  * connection.c -- defines the connection object DBAPI 2.0
- * $Id: connection.c 554 2004-10-30 00:19:27Z fog $
+ * $Id: connection.c 884 2005-09-12 09:20:17Z fog $
  */
 
 #include "module.h"
@@ -50,6 +50,10 @@ curs_closeall(connobject *self)
 }
 
 
+typedef struct {
+    cursobject *cursor;
+    char *errmsg;
+} doall_state_t;
 
 /* _curs_doall() - execute an operation (commit or rollback) on all the cursors 
  * 
@@ -65,21 +69,37 @@ _curs_doall(connobject *self, int (*operation)(cursobject *) )
     int len, i, has_errors = 0;
     cursobject *cursor;
 
-    PyObject* cursors = self->cursors;
-    PyObject* errs = PyDict_New();
+    doall_state_t *cursors = NULL;
+    PyObject* errs = NULL;
 
     Dprintf("curs_doall: acquiring lock\n");
     pthread_mutex_lock(&(self->lock));
     Dprintf("curs_doall: lock acquired\n");
+
+    /* collect all the cursors, so we can use them while not holding the GIL
+     * We keep a reference to the cursors, in case the self->cursors changes
+     * during the call. */
+    len = PyList_Size(self->cursors);
+    cursors = (doall_state_t *)malloc(len * sizeof (doall_state_t));
+    if (!cursors) {
+        pthread_mutex_unlock(&(self->lock));
+        Dprintf("curs_doall: lock released\n");
+        return PyErr_NoMemory();
+    }
+    for (i = 0; i < len; i++) {
+        cursors[i].cursor = (cursobject *)PyList_GetItem(self->cursors, i);
+        assert(cursors[i].cursor);
+        Py_INCREF(cursors[i].cursor);
+        cursors[i].errmsg = NULL;
+    }
+
     Py_BEGIN_ALLOW_THREADS;
     
-    len = PyList_Size(cursors);
     Dprintf("curs_doall: %d cursors\n", len);
     
     /* acquire all the required locks */
     for (i = 0; i < len; i++) {
-        cursor = (cursobject *)PyList_GetItem(cursors, i);
-        assert(cursor);
+        cursor = cursors[i].cursor;
         Dprintf("curs_doall: lock/iterating on %p\n", cursor);
         if (cursor->keeper->status == KEEPER_BEGIN
             && cursor->isolation_level > 0){
@@ -98,8 +118,8 @@ _curs_doall(connobject *self, int (*operation)(cursobject *) )
     /* does all the operations */
     for (i = 0; i < len; i++) {
         int status = 0;
-        cursor = (cursobject *)PyList_GetItem(cursors, i);
-        assert(cursor);
+
+        cursor = cursors[i].cursor;
         Dprintf("curs_doall: iterating on %p\n", cursor);
         if (cursor->keeper->status == KEEPER_CONN_LOCK) {
             Dprintf("curs_doall: operating on cursor %p\n", cursor);
@@ -107,10 +127,8 @@ _curs_doall(connobject *self, int (*operation)(cursobject *) )
             status = (*operation)(cursor);
             if (status == -1) {
                 has_errors = 1;
-                if (errs) {
-                    PyObject* str = PyString_FromString(cursor->critical);
-                    PyDict_SetItem(errs, (PyObject *)cursor, str);
-                    Py_XDECREF(str);
+                if (cursor->critical) {
+                    cursors[i].errmsg = strdup(cursor->critical);
                 }
             }
             cursor->keeper->status = KEEPER_CONN_READY;
@@ -119,8 +137,7 @@ _curs_doall(connobject *self, int (*operation)(cursobject *) )
 
     /* unlocks all the connections */
     for (i = 0; i < len; i++) {
-        cursor = (cursobject *)PyList_GetItem(cursors, i);
-        assert(cursor);
+        cursor = cursors[i].cursor;
         if (cursor->keeper->status == KEEPER_CONN_READY) {
             pthread_mutex_unlock(&(cursor->keeper->lock));
             cursor->keeper->status = KEEPER_READY;
@@ -133,18 +150,34 @@ _curs_doall(connobject *self, int (*operation)(cursobject *) )
     Dprintf("curs_doall: lock released\n");
     Py_END_ALLOW_THREADS;
 
+    /* if an error occurred, set up the error dictionary (or set errs to
+     * None if we can't create the dictionary). */
     if (has_errors) {
-        if (errs)
-            return errs;
-        else {
-            Py_INCREF(Py_None);
-            return Py_None;
-        }
+        errs = PyDict_New();
+        if (errs) {
+            for (i = 0; i < len; i++) {
+                if (cursors[i].errmsg != NULL) {
+                    PyObject *str = PyString_FromString(cursors[i].errmsg);
+                    PyDict_SetItem(errs, (PyObject *)cursors[i].cursor, str);
+                    Py_XDECREF(str);
+                }
+            }
+        } else {
+	  errs = Py_None;
+	  Py_INCREF(errs);
+	}
     }
-    else {
-        Py_DECREF(errs);
-        return NULL;
+
+    /* clean up the state array */
+    for (i = 0; i < len; i++) {
+        Py_DECREF(cursors[i].cursor);
+        if (cursors[i].errmsg)
+            free(cursors[i].errmsg);
     }
+    free(cursors);
+
+    /* errs will be NULL if has_errors is False */
+    return errs;
 }
 
 
@@ -232,12 +265,15 @@ _psyco_conn_close(connobject *self)
     
     Py_DECREF(self->cursors);
     Py_DECREF(self->avail_conn);
+    self->cursors = NULL;
+    self->avail_conn = NULL;
 
     /* orphan default cursor and destroy it (closing the last connection to the
        database) */
     Dprintf("_psyco_conn_close(): killing stdmanager\n");
     self->stdmanager->conn = NULL;
     Py_DECREF(self->stdmanager);
+    self->stdmanager = NULL;
 }
 
 static PyObject *
@@ -364,7 +400,7 @@ psyco_conn_cursor(connobject *self, PyObject *args)
     
     EXC_IFCLOSED(self);
 
-    Dprintf("psyco_conn_cursor(): conn = %p, name = %s\n", self, name);
+    Dprintf("psyco_conn_cursor(): conn = %p\n", self);
     Dprintf("psyco_conn_cursor(): serialize = %d, keeper = %p\n",
             self->serialize, self->stdmanager->keeper);
     
@@ -427,7 +463,9 @@ psyco_conn_set_isolation_level(connobject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "l", &level)) {
         return NULL;
     }
-    
+
+    EXC_IFCLOSED(self);
+
     _psyco_conn_set_isolation_level(self, level);
     
     Py_INCREF(Py_None);
@@ -451,6 +489,9 @@ psyco_conn_autocommit(connobject *self, PyObject *args)
     }
 
     if (ac == 0) isolation_level = 2;
+
+    EXC_IFCLOSED(self);
+
     _psyco_conn_set_isolation_level(self, isolation_level);
 
     Py_INCREF(Py_None);
@@ -471,6 +512,8 @@ psyco_conn_serialize(connobject *self, PyObject *args)
         return NULL;
     }
 
+    EXC_IFCLOSED(self);
+
     /* save the value */
     self->serialize = (int)se;
 
