@@ -21,7 +21,7 @@
 
 /*
  * cursor.c - defines the cursor object DBAPI 2.0
- * $Id: cursor.c 577 2004-11-17 11:42:24Z fog $
+ * $Id: cursor.c 826 2005-07-16 19:04:16Z fog $
  */
 
 #ifdef _WIN32
@@ -436,7 +436,7 @@ alloc_keeper(connobject *conn)
     /* sets the notice processor callback */
     PQsetNoticeProcessor(pgconn, pgconn_notice_callback, (void*)conn);
 
-#if POSTGRESQL_MINOR >= 4   
+#if POSTGRESQL_MAJOR >= 8 || POSTGRESQL_MINOR >= 4   
     Dprintf("alloc_keeper: connection protocol version: %d\n",
             PQprotocolVersion(pgconn));
 #endif
@@ -897,14 +897,15 @@ _psyco_curs_execute(cursobject *self, char *query,
             }
 
             if (ftype == NUMERICOID) {
-                /* scale */
+                /* precision */
                 PyTuple_SET_ITEM(dtitem, 4,
                                  PyInt_FromLong((fmod >> 16) & 0xFFFF));
 
-                /* precision */
+                /* scale */
                 PyTuple_SET_ITEM(dtitem, 5,
-                                 PyInt_FromLong(fmod & 0xFFFF));
+                                 PyInt_FromLong((fmod & 0xFFFF) - 4));
             }
+
             else {
                 /* scale */
                 Py_INCREF(Py_None);
@@ -945,7 +946,7 @@ _psyco_curs_execute(cursobject *self, char *query,
         char *errstr = NULL;
 
         if (asprintf(&errstr, "%s\n%s", pgerr, query) >= 0) {
-#if POSTGRESQL_MINOR >= 4
+#if POSTGRESQL_MAJOR >= 8 || POSTGRESQL_MINOR >= 4
             char *pgstate = PQresultErrorField(self->pgres, PG_DIAG_SQLSTATE);
 
             /* if pgstate is NULL we are using a new libpq to connect to an
@@ -1579,7 +1580,7 @@ psyco_curs_dictfetchone(cursobject *self, PyObject *args)
     PyObject *value = NULL;
     
     res = psyco_curs_fetchone(self, args);
-    if (res == Py_None) return res;
+    if (res == NULL || res == Py_None) return res;
 
     dict = PyDict_New();
     for (i=0; i<self->columns; i++) {
