@@ -16,7 +16,7 @@
 
 ## put in DSN your DSN string
 
-DSN = 'dbname=test user=fog'
+DSN = 'dbname=test user=test'
 
 ## don't modify anything below tis line (except for experimenting)
 
@@ -32,7 +32,6 @@ curs = conn.cursor()
 try:
     curs.execute("CREATE TABLE test_cast (p1 point, p2 point, b box)")
 except:
-    conn.rollback()
     curs.execute("DROP TABLE test_cast")
     curs.execute("CREATE TABLE test_cast (p1 point, p2 point, b box)")
 conn.commit()
