@@ -20,7 +20,7 @@
 
 /*
  * typemod.c -- types available to the psycopg module
- * $Id: typemod.c 554 2004-10-30 00:19:27Z fog $
+ * $Id: typemod.c 839 2005-08-22 22:55:21Z fog $
  */
 
 #include "module.h"
@@ -43,17 +43,38 @@ static struct memberlist psyco_DateTimeObject_memberlist[] = {
 
 /* the python object interface for the date and time object */
 
-static char *psyco_DateTimeObject_str_conv[] = {
-    PSYCO_DATETIME_TIME_CONV,
-    PSYCO_DATETIME_DATE_CONV,
-    PSYCO_DATETIME_TIMESTAMP_CONV
-};
-
 static PyObject *
 psyco_DateTimeObject_str(psyco_DateTimeObject *self)
 {
-    return PyObject_CallMethod(self->datetime, "strftime", "s",
-                               psyco_DateTimeObject_str_conv[self->type]);
+    PyObject *res = NULL;
+    char *buffer = NULL;
+    
+    mxDateTimeObject *obj = (mxDateTimeObject*)self->datetime;
+
+    switch (self->type) {
+        
+    case 0:
+        asprintf(&buffer, "'%02d:%02d:%.6f'",
+                 (int)obj->hour, (int)obj->minute, (float)obj->second);
+        if (buffer) res = PyString_FromString(buffer);
+        break;
+
+    case 1:
+        asprintf(&buffer, "'%ld-%02d-%02d'",
+                 obj->year, (int)obj->month, (int)obj->day);
+        if (buffer) res = PyString_FromString(buffer);
+        break;
+
+    case 2:
+        asprintf(&buffer, "'%ld-%02d-%02d %02d:%02d:%.6f'",
+                 obj->year, (int)obj->month, (int)obj->day,
+                 (int)obj->hour, (int)obj->minute, (float)obj->second);
+        if (buffer) res = PyString_FromString(buffer);
+        break;
+    }
+
+    if (buffer) free(buffer);
+    return res;
 }
 
 static struct PyMethodDef psyco_DateTimeObject_methods[] = {
@@ -236,7 +257,7 @@ new_psyco_bufferobject(PyObject *buffer)
     obj = PyObject_NEW(psyco_BufferObject, &psyco_BufferObject_Type);
     if (obj == NULL) return NULL;
 
-    original = PyString_AS_STRING(buffer);
+    original = (unsigned char*)PyString_AS_STRING(buffer);
     len = PyString_GET_SIZE(buffer);
     space = len + 2;
 
@@ -309,7 +330,8 @@ new_psyco_bufferobject(PyObject *buffer)
     Py_END_ALLOW_THREADS;
 
     Dprintf("new_psyco_bufferobject(): quoted = %s\n", quoted);
-    obj->buffer = PyString_FromStringAndSize(quoted, chptr - quoted + 1 );
+    obj->buffer = PyString_FromStringAndSize((char*)quoted, 
+                                             chptr - quoted + 1 );
     free(quoted);
     
     Dprintf("new_psyco_bufferobject: object created at %p\n", obj);
