@@ -6,7 +6,7 @@ static int psyco_cast_types_STRING[] = {19, 18, 25, 1042, 1043, 0};
 static int psyco_cast_types_BOOLEAN[] = {16, 0};
 static int psyco_cast_types_DATETIME[] = {1082, 1083, 1266, 1114, 1184, 704, 1186, 0};
 static int psyco_cast_types_TIME[] = {1083, 1266, 0};
-static int psyco_cast_types_DATE[] = {1082, 1114, 0};
+static int psyco_cast_types_DATE[] = {1082, 1114, 1184, 0};
 static int psyco_cast_types_INTERVAL[] = {704, 1186, 0};
 static int psyco_cast_types_BINARY[] = {17, 0};
 static int psyco_cast_types_ROWID[] = {26, 0};
